// THeadLabel Version 1.0
// written 1997 by Marc Hoffmann
// Status: Freeware

unit headlbl;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls;

type
  TNumGlyphs = 1..4;

  // define new component
  THeadLabel = class(TCustomLabel)
  private
    { Private-Deklarationen }
    FLeftSpacing: Byte;
    FRightSpacing: Byte;
    FSubCaption: String;
    FSubStyle: TFontStyles;
    FSubEllipsis: Boolean;
    FFontColor: TColor;
    FGlyph: TBitmap;
    FNumGlyphs: TNumGlyphs;
    procedure SetLeftSpacing(Value: Byte);
    procedure SetRightSpacing(Value: Byte);
    procedure SetSubCaption(Value: String);
    procedure SetSubStyle(Value: TFontStyles);
    procedure SetSubEllipsis(Value: Boolean);
    procedure SetFontColor(Value: TColor);
    procedure SetGlyph(Value: TBitmap);
    procedure SetNumGlyphs(Value: TNumGlyphs);
  protected
    { Protected-Deklarationen }
    procedure Paint; override;
  public
    { Public-Deklarationen }
    constructor Create(AOwner: TComponent); override;
    destructor  Destroy; override;
  published
    { Published-Deklarationen }
    property LeftSpacing: Byte read FLeftSpacing write SetLeftSpacing default 5;
    property RightSpacing: Byte read FRightSpacing write SetRightSpacing default 8;
    property SubCaption: String read FSubCaption write SetSubCaption;
    property SubStyle: TFontStyles read FSubStyle write SetSubStyle;
    property SubEllipsis: Boolean read FSubEllipsis write SetSubEllipsis;
    property FontColor: TColor read FFontColor write SetFontColor default clBlack;
    property Glyph: TBitmap read FGlyph write SetGlyph;
    property NumGlyphs: TNumGlyphs read FNumGlyphs write SetNumGlyphs default 1;
    property ShowHint;
    property ParentShowHint;
    property ParentColor;
    property Color;
    property Caption;
    property FocusControl;
    property PopupMenu;
    property OnClick;
    property OnDblClick;
  end;

procedure Register;

implementation

// create new component-instance & set defaults
constructor THeadLabel.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FGlyph:=TBitmap.Create;
  FGlyph.Assign(nil);
  FNumGlyphs:=1;
  AutoSize:=False;
  Font.Name:='ARIAL';
  Color:=clGray;
  Height:=20;
  Width:=377;
  FLeftSpacing:=5;
  FRightSpacing:=8;
  FSubEllipsis:=False;
  FFontColor:=clBlack;
end;

// destroy new component & free memory allocations (used by the glyph)
destructor THeadLabel.Destroy;
begin
  FGlyph.Free;
  inherited Destroy;
end;

// set left spacing
procedure THeadLabel.SetLeftSpacing(Value: Byte);
begin
  if Value <> FLeftSpacing then begin
    FLeftSpacing:=Value;
    invalidate;
  end;
end;

// set right spacing
procedure THeadLabel.SetRightSpacing(Value: Byte);
begin
  if Value <> FRightSpacing then begin
    FRightSpacing:=Value;
    invalidate;
  end;
end;

// set subcaption
procedure THeadLabel.SetSubCaption(Value: String);
begin
  if Value <> FSubCaption then begin
    FSubCaption:=Value;
    invalidate;
  end;
end;

// set substyle
procedure THeadLabel.SetSubStyle(Value: TFontStyles);
begin
  if Value <> FSubStyle then begin
    FSubStyle:=Value;
    invalidate;
  end;
end;

// set subellipsis
procedure THeadLabel.SetSubEllipsis(Value: Boolean);
begin
  if Value <> FSubEllipsis then begin
    FSubEllipsis:=Value;
    invalidate;
  end;
end;

// set fontcolor
procedure THeadLabel.SetFontColor(Value: TColor);
begin
  if Value <> FFontColor then begin
    FFontColor:=Value;
    invalidate;
  end;
end;

// set glyph
procedure THeadLabel.SetGlyph(Value: TBitmap);
begin
  if Value <> FGlyph then begin
    FGlyph.Assign(Value);
    invalidate;
  end;
end;

// set numglyphs
procedure THeadLabel.SetNumGlyphs(Value: TNumGlyphs);
begin
  if Value <> FNumGlyphs then begin
    FNumGlyphs:=Value;
    invalidate;
  end;
end;

// paint component
procedure THeadLabel.Paint;
var
  Rect: TRect;
  Dummy: String;
  Bitmap: TBitmap;
  GlyphOffset: Integer;
begin
  with Canvas do begin
    // fill clientrect
    Brush.Color:=Color;
    FillRect(ClientRect);

    // display transparent glyph
    Bitmap:=TBitmap.Create;
    try
      Bitmap.Assign(FGlyph);
      GlyphOffset:=0;
      if Assigned(Bitmap) then begin
        if Bitmap.Height > Height then Bitmap.Height:= Height - 2;
        Bitmap.Width:=Bitmap.Width div FNumGlyphs;
        BrushCopy(Bounds(LeftSpacing, (Height - Bitmap.Height) div 2, Bitmap.Width, Bitmap.Height), Bitmap, Bounds(0, 0, Bitmap.Width, Bitmap.Height), Bitmap.TransparentColor);
        GlyphOffset:=3 + Bitmap.Width;
      end;
    finally
      Bitmap.Free;
    end;

    // display main-caption (left alignment)
    Font.Color:=FFontColor;
    Font.Style:=[fsBold];
    Font.Size:=10;
    Rect:=ClientRect;
    Inc(Rect.Left, GlyphOffset + LeftSpacing);
    Rect.Top:=(Height - TextHeight(Caption)) div 2;
    DrawText(Handle, PChar(Caption), Length(Caption), Rect, DT_LEFT);

    // display sub-caption (right alignment)
    if FSubCaption <> '' then begin
      Dummy:=FSubCaption;
      if FSubEllipsis then Dummy:=Concat(Dummy, '...');
      Font.Size:=8;
      Font.Style:=FSubStyle;
      Rect:=ClientRect;
      Rect.Top:=(Height - TextHeight(Dummy)) div 2;
      Dec(Rect.Right, RightSpacing);
      DrawText(Handle, PChar(Dummy), Length(Dummy), Rect, DT_RIGHT);
    end;
  end;
end;

// register new component
procedure Register;
begin
  RegisterComponents('Freeware', [THeadLabel]);
end;

end.
